@@ -41,6 +41,8 @@
 #ifdef ENABLE_OPENPGP
 #include "openpgp/gnutls_openpgp.h"
 #endif
+#include "gettext.h"
+#define _(String) dgettext (PACKAGE, String)
 
 /**
  * gnutls_certificate_free_keys:
@@ -193,6 +195,7 @@ gnutls_certificate_free_credentials (gnutls_certificate_credentials_t sc)
   gnutls_x509_trust_list_deinit(sc->tlist, 1);
   gnutls_certificate_free_keys (sc);
   gnutls_certificate_free_ca_names (sc);
+  gnutls_free(sc->ocsp_response_file);
 
 #ifdef ENABLE_OPENPGP
   gnutls_openpgp_keyring_deinit (sc->keyring);
@@ -231,7 +234,6 @@ int ret;
     }
   (*res)->verify_bits = DEFAULT_MAX_VERIFY_BITS;
   (*res)->verify_depth = DEFAULT_MAX_VERIFY_DEPTH;
-  (*res)->verify_flags = GNUTLS_VERIFY_ALLOW_UNSORTED_CHAIN;
 
   return 0;
 }
@@ -580,6 +582,7 @@ _gnutls_x509_get_raw_crt_expiration_time (const gnutls_datum_t * cert)
  -*/
 static int
 _gnutls_openpgp_crt_verify_peers (gnutls_session_t session,
+                                  const char* hostname,
                                   unsigned int *status)
 {
   cert_auth_info_t info;
@@ -593,7 +596,7 @@ _gnutls_openpgp_crt_verify_peers (gnutls_session_t session,
     return GNUTLS_E_INVALID_REQUEST;
 
   cred = (gnutls_certificate_credentials_t)
-    _gnutls_get_cred (session->key, GNUTLS_CRD_CERTIFICATE, NULL);
+    _gnutls_get_cred (session, GNUTLS_CRD_CERTIFICATE, NULL);
   if (cred == NULL)
     {
       gnutls_assert ();
@@ -620,7 +623,7 @@ _gnutls_openpgp_crt_verify_peers (gnutls_session_t session,
   /* Verify certificate 
    */
   ret =
-    _gnutls_openpgp_verify_key (cred, &info->raw_certificate_list[0],
+    _gnutls_openpgp_verify_key (cred, hostname, &info->raw_certificate_list[0],
                                 peer_certificate_list_size, status);
 
   if (ret < 0)
@@ -628,7 +631,7 @@ _gnutls_openpgp_crt_verify_peers (gnutls_session_t session,
       gnutls_assert ();
       return ret;
     }
-
+    
   return 0;
 }
 #endif
@@ -638,16 +641,22 @@ _gnutls_openpgp_crt_verify_peers (gnutls_session_t session,
  * @session: is a gnutls session
  * @status: is the output of the verification
  *
- * This function will try to verify the peer's certificate and return
- * its status (trusted, invalid etc.).  The value of @status should
- * be one or more of the gnutls_certificate_status_t enumerated
- * elements bitwise or'd. To avoid denial of service attacks some
+ * This function will verify the peer's certificate and store
+ * the status in the @status variable as a bitwise or'd gnutls_certificate_status_t
+ * values or zero if the certificate is trusted. Note that value in @status
+ * is set only when the return value of this function is success (i.e, failure 
+ * to trust a certificate does not imply a negative return value).
+ *
+ * If available the OCSP Certificate Status extension will be
+ * utilized by this function.
+ * 
+ * To avoid denial of service attacks some
  * default upper limits regarding the certificate key size and chain
- * size are set. To override them use
- * gnutls_certificate_set_verify_limits().
+ * size are set. To override them use gnutls_certificate_set_verify_limits().
  *
  * Note that you must also check the peer's name in order to check if
- * the verified certificate belongs to the actual peer, see gnutls_x509_crt_check_hostname().
+ * the verified certificate belongs to the actual peer, see gnutls_x509_crt_check_hostname(),
+ * or use gnutls_certificate_verify_peers3().
  *
  * Returns: a negative error code on error and %GNUTLS_E_SUCCESS (0) on success.
  **/
@@ -671,10 +680,68 @@ gnutls_certificate_verify_peers2 (gnutls_session_t session,
   switch (gnutls_certificate_type_get (session))
     {
     case GNUTLS_CRT_X509:
-      return _gnutls_x509_cert_verify_peers (session, status);
+      return _gnutls_x509_cert_verify_peers (session, NULL, status);
 #ifdef ENABLE_OPENPGP
     case GNUTLS_CRT_OPENPGP:
-      return _gnutls_openpgp_crt_verify_peers (session, status);
+      return _gnutls_openpgp_crt_verify_peers (session, NULL, status);
+#endif
+    default:
+      return GNUTLS_E_INVALID_REQUEST;
+    }
+}
+
+/**
+ * gnutls_certificate_verify_peers3:
+ * @session: is a gnutls session
+ * @hostname: is the expected name of the peer; may be %NULL
+ * @status: is the output of the verification
+ *
+ * This function will verify the peer's certificate and store the
+ * status in the @status variable as a bitwise or'd gnutls_certificate_status_t
+ * values or zero if the certificate is trusted. Note that value in @status
+ * is set only when the return value of this function is success (i.e, failure 
+ * to trust a certificate does not imply a negative return value).
+ *
+ * If the @hostname provided is non-NULL then this function will compare
+ * the hostname in the certificate against the given. If they do not match 
+ * the %GNUTLS_CERT_UNEXPECTED_OWNER status flag will be set.
+ *
+ * If available the OCSP Certificate Status extension will be
+ * utilized by this function.
+ * 
+ * To avoid denial of service attacks some
+ * default upper limits regarding the certificate key size and chain
+ * size are set. To override them use gnutls_certificate_set_verify_limits().
+ *
+ * Returns: a negative error code on error and %GNUTLS_E_SUCCESS (0) on success.
+ *
+ * Since: 3.1.4
+ **/
+int
+gnutls_certificate_verify_peers3 (gnutls_session_t session,
+                                  const char* hostname,
+                                  unsigned int *status)
+{
+  cert_auth_info_t info;
+
+  CHECK_AUTH (GNUTLS_CRD_CERTIFICATE, GNUTLS_E_INVALID_REQUEST);
+
+  info = _gnutls_get_auth_info (session);
+  if (info == NULL)
+    {
+      return GNUTLS_E_NO_CERTIFICATE_FOUND;
+    }
+
+  if (info->raw_certificate_list == NULL || info->ncerts == 0)
+    return GNUTLS_E_NO_CERTIFICATE_FOUND;
+
+  switch (gnutls_certificate_type_get (session))
+    {
+    case GNUTLS_CRT_X509:
+      return _gnutls_x509_cert_verify_peers (session, hostname, status);
+#ifdef ENABLE_OPENPGP
+    case GNUTLS_CRT_OPENPGP:
+      return _gnutls_openpgp_crt_verify_peers (session, hostname, status);
 #endif
     default:
       return GNUTLS_E_INVALID_REQUEST;
@@ -840,3 +907,87 @@ _gnutls_check_key_cert_match (gnutls_certificate_credentials_t res)
 
   return 0;
 }
+
+/**
+ * gnutls_certificate_verification_status_print:
+ * @status: The status flags to be printed
+ * @type: The certificate type
+ * @out: Newly allocated datum with (0) terminated string.
+ * @flags: should be zero
+ *
+ * This function will pretty print the status of a verification
+ * process -- eg. the one obtained by gnutls_certificate_verify_peers3().
+ *
+ * The output @out needs to be deallocated using gnutls_free().
+ *
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
+ *   negative error value.
+ *
+ * Since: 3.1.4
+ **/
+int
+gnutls_certificate_verification_status_print (unsigned int status,
+                       gnutls_certificate_type_t type,
+                       gnutls_datum_t * out, unsigned int flags)
+{
+  gnutls_buffer_st str;
+  int ret;
+
+  _gnutls_buffer_init (&str);
+
+  if (status == 0)
+    _gnutls_buffer_append_str (&str, _("The certificate is trusted. "));
+  else
+    _gnutls_buffer_append_str (&str, _("The certificate is NOT trusted. "));
+
+  if (type == GNUTLS_CRT_X509)
+    {
+      if (status & GNUTLS_CERT_REVOKED)
+        _gnutls_buffer_append_str (&str, _("The certificate chain revoked. "));
+
+      if (status & GNUTLS_CERT_REVOCATION_DATA_SUPERSEDED)
+         _gnutls_buffer_append_str (&str, _("The revocation data are old and have been superseded. "));
+
+      if (status & GNUTLS_CERT_REVOCATION_DATA_ISSUED_IN_FUTURE)
+         _gnutls_buffer_append_str (&str, _("The revocation data are issued with a future date. "));
+
+      if (status & GNUTLS_CERT_SIGNER_NOT_FOUND)
+         _gnutls_buffer_append_str (&str, _("The certificate issuer is unknown. "));
+
+      if (status & GNUTLS_CERT_SIGNER_NOT_CA)
+         _gnutls_buffer_append_str (&str, _("The certificate issuer is not a CA. "));
+      }
+    else if (type == GNUTLS_CRT_OPENPGP)
+      {
+        _gnutls_buffer_append_str (&str, _("The certificate is not trusted. "));
+
+        if (status & GNUTLS_CERT_SIGNER_NOT_FOUND)
+          _gnutls_buffer_append_str (&str, _("Could not find a signer of the certificate. "));
+
+        if (status & GNUTLS_CERT_REVOKED)
+          _gnutls_buffer_append_str (&str, _("The certificate is revoked. "));
+      }
+
+  if (status & GNUTLS_CERT_INSECURE_ALGORITHM)
+    _gnutls_buffer_append_str (&str, _("The certificate chain uses insecure algorithm. "));
+
+  if (status & GNUTLS_CERT_SIGNER_CONSTRAINTS_FAILURE)
+    _gnutls_buffer_append_str (&str, _("The certificate chain violates the signer's constraints. "));
+
+  if (status & GNUTLS_CERT_NOT_ACTIVATED)
+    _gnutls_buffer_append_str (&str, _("The certificate chain uses not yet valid certificate. "));
+
+  if (status & GNUTLS_CERT_EXPIRED)
+    _gnutls_buffer_append_str (&str, _("The certificate chain uses expired certificate. "));
+
+  if (status & GNUTLS_CERT_SIGNATURE_FAILURE)
+    _gnutls_buffer_append_str (&str, _("The signature in the certificate is invalid. "));
+
+  if (status & GNUTLS_CERT_UNEXPECTED_OWNER)
+    _gnutls_buffer_append_str (&str, _("The name in the certificate does not match the expected. "));
+
+  ret = _gnutls_buffer_to_datum( &str, out);
+  if (out->size > 0) out->size--;
+      
+  return ret;
+}
