@@ -89,11 +89,11 @@ _set_priority (priority_st * st, const int *list)
 inline static void
 _add_priority (priority_st * st, const int *list)
 {
-  int num = 0, i, j, init;
+  int num, i, j, init;
 
   init = i = st->algorithms;
 
-  while (list[num] != 0)
+  for (num=0;list[num]!=0;++num)
     {
       if (i+1 > MAX_ALGOS)
         {
@@ -104,14 +104,15 @@ _add_priority (priority_st * st, const int *list)
         {
           if (st->priority[j] == (unsigned)list[num])
             {
-              num++;
-              continue;
+              break;
             }
         }
 
-      st->priority[i++] = list[num];
-      st->algorithms++;
-      num++;
+      if (j == init)
+        {
+          st->priority[i++] = list[num];
+          st->algorithms++;
+        }
     }
     
   return;
@@ -995,6 +996,12 @@ gnutls_priority_init (gnutls_priority_t * priority_cache,
               (*priority_cache)->additional_verify_flags |=
                 GNUTLS_VERIFY_ALLOW_SIGN_RSA_MD5;
             }
+          else if (strcasecmp (&broken_list[i][1],
+                               "VERIFY_DISABLE_CRL_CHECKS") == 0)
+            {
+              (*priority_cache)->additional_verify_flags |=
+                GNUTLS_VERIFY_DISABLE_CRL_CHECKS;
+            }
           else if (strcasecmp (&broken_list[i][1],
                                "SSL3_RECORD_VERSION") == 0)
             (*priority_cache)->ssl3_record_version = 1;
