@@ -117,6 +117,8 @@ const ASN1_ARRAY_TYPE pkix_asn1_tab[] = {
   { NULL, 4360, "30"},
   { "UTF8String", 1610620935, NULL },
   { NULL, 4360, "12"},
+  { "VisibleString", 1610620935, NULL },
+  { NULL, 4360, "26"},
   { "Attribute", 1610612741, NULL },
   { "type", 1073741826, "AttributeType"},
   { "values", 536870927, NULL },
@@ -388,6 +390,35 @@ const ASN1_ARRAY_TYPE pkix_asn1_tab[] = {
   { "ProxyPolicy", 1610612741, NULL },
   { "policyLanguage", 1073741836, NULL },
   { "policy", 16391, NULL },
+  { "certificatePolicies", 1612709899, NULL },
+  { "MAX", 1074266122, "1"},
+  { NULL, 2, "PolicyInformation"},
+  { "PolicyInformation", 1610612741, NULL },
+  { "policyIdentifier", 1073741836, NULL },
+  { "policyQualifiers", 538984459, NULL },
+  { "MAX", 1074266122, "1"},
+  { NULL, 2, "PolicyQualifierInfo"},
+  { "PolicyQualifierInfo", 1610612741, NULL },
+  { "policyQualifierId", 1073741836, NULL },
+  { "qualifier", 541065229, NULL },
+  { "policyQualifierId", 1, NULL },
+  { "CPSuri", 1073741826, "IA5String"},
+  { "UserNotice", 1610612741, NULL },
+  { "noticeRef", 1073758210, "NoticeReference"},
+  { "explicitText", 16386, "DisplayText"},
+  { "NoticeReference", 1610612741, NULL },
+  { "organization", 1073741826, "DisplayText"},
+  { "noticeNumbers", 536870923, NULL },
+  { NULL, 3, NULL },
+  { "DisplayText", 1610612754, NULL },
+  { "ia5String", 1612709890, "IA5String"},
+  { "200", 524298, "1"},
+  { "visibleString", 1612709890, "VisibleString"},
+  { "200", 524298, "1"},
+  { "bmpString", 1612709890, "BMPString"},
+  { "200", 524298, "1"},
+  { "utf8String", 538968066, "UTF8String"},
+  { "200", 524298, "1"},
   { "OCSPRequest", 1610612741, NULL },
   { "tbsRequest", 1073741826, "TBSRequest"},
   { "optionalSignature", 536895490, "Signature"},
