@@ -126,26 +126,19 @@ gnutls_x509_crq_import (gnutls_x509_crq_t crq,
    */
   if (format == GNUTLS_X509_FMT_PEM)
     {
-      uint8_t *out;
-
       /* Try the first header */
-      result = _gnutls_fbase64_decode (PEM_CRQ, data->data, data->size, &out);
+      result = _gnutls_fbase64_decode (PEM_CRQ, data->data, data->size, &_data);
 
-      if (result <= 0)          /* Go for the second header */
+      if (result < 0)          /* Go for the second header */
         result =
-          _gnutls_fbase64_decode (PEM_CRQ2, data->data, data->size, &out);
+          _gnutls_fbase64_decode (PEM_CRQ2, data->data, data->size, &_data);
 
-      if (result <= 0)
+      if (result < 0)
         {
-          if (result == 0)
-            result = GNUTLS_E_INTERNAL_ERROR;
           gnutls_assert ();
           return result;
         }
 
-      _data.data = out;
-      _data.size = result;
-
       need_free = 1;
     }
 
@@ -268,7 +261,7 @@ gnutls_x509_crq_get_dn (gnutls_x509_crq_t crq, char *buf, size_t * sizeof_buf)
  * @oid: holds an Object Identified in null terminated string
  * @indx: In case multiple same OIDs exist in the RDN, this specifies
  *   which to send. Use (0) to get the first one.
- * @raw_flag: If non (0) returns the raw DER data of the DN part.
+ * @raw_flag: If non-zero returns the raw DER data of the DN part.
  * @buf: a pointer to a structure to hold the name (may be %NULL)
  * @sizeof_buf: initially holds the size of @buf
  *
@@ -1162,6 +1155,38 @@ gnutls_x509_crq_export (gnutls_x509_crq_t crq,
                                   output_data, output_data_size);
 }
 
+/**
+ * gnutls_x509_crq_export2:
+ * @crq: should contain a #gnutls_x509_crq_t structure
+ * @format: the format of output params. One of PEM or DER.
+ * @out: will contain a certificate request PEM or DER encoded
+ *
+ * This function will export the certificate request to a PEM or DER
+ * encoded PKCS10 structure.
+ *
+ * The output buffer is allocated using gnutls_malloc().
+ *
+ * If the structure is PEM encoded, it will have a header of "BEGIN
+ * NEW CERTIFICATE REQUEST".
+ *
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
+ *   negative error value.
+ *
+ * Since 3.1.3
+ **/
+int
+gnutls_x509_crq_export2 (gnutls_x509_crq_t crq,
+                        gnutls_x509_crt_fmt_t format, gnutls_datum_t *out)
+{
+  if (crq == NULL)
+    {
+      gnutls_assert ();
+      return GNUTLS_E_INVALID_REQUEST;
+    }
+
+  return _gnutls_x509_export_int2 (crq->crq, format, PEM_CRQ, out);
+}
+
 /**
  * gnutls_x509_crq_get_pk_algorithm:
  * @crq: should contain a #gnutls_x509_crq_t structure
@@ -1547,7 +1572,7 @@ gnutls_x509_crq_get_extension_data (gnutls_x509_crq_t crq, int indx,
  * gnutls_x509_crq_get_key_usage:
  * @crq: should contain a #gnutls_x509_crq_t structure
  * @key_usage: where the key usage bits will be stored
- * @critical: will be non (0) if the extension is marked as critical
+ * @critical: will be non-zero if the extension is marked as critical
  *
  * This function will return certificate's key usage, by reading the
  * keyUsage X.509 extension (2.5.29.15).  The key usage value will
@@ -1604,7 +1629,7 @@ gnutls_x509_crq_get_key_usage (gnutls_x509_crq_t crq,
 /**
  * gnutls_x509_crq_get_basic_constraints:
  * @crq: should contain a #gnutls_x509_crq_t structure
- * @critical: will be non (0) if the extension is marked as critical
+ * @critical: will be non-zero if the extension is marked as critical
  * @ca: pointer to output integer indicating CA status, may be NULL,
  *   value is 1 if the certificate CA flag is set, 0 otherwise.
  * @pathlen: pointer to output integer indicating path length (may be
@@ -1752,7 +1777,7 @@ get_subject_alt_name (gnutls_x509_crq_t crq,
  * @ret: is the place where the alternative name will be copied to
  * @ret_size: holds the size of ret.
  * @ret_type: holds the #gnutls_x509_subject_alt_name_t name type
- * @critical: will be non (0) if the extension is marked as critical
+ * @critical: will be non-zero if the extension is marked as critical
  *   (may be null)
  *
  * This function will return the alternative names, contained in the
@@ -1827,7 +1852,7 @@ gnutls_x509_crq_get_subject_alt_othername_oid (gnutls_x509_crq_t crq,
  *   specifies which to send. Use (0) to get the first one.
  * @buf: a pointer to a structure to hold the name (may be null)
  * @sizeof_buf: initially holds the size of @buf
- * @critical: will be non (0) if the extension is marked as critical
+ * @critical: will be non-zero if the extension is marked as critical
  *
  * This function will return the extension specified by the OID in
  * the certificate.  The extensions will be returned as binary data
@@ -2334,7 +2359,7 @@ gnutls_x509_crq_set_key_purpose_oid (gnutls_x509_crq_t crq,
  * @output_data_size: holds the size of output_data (and will be
  *   replaced by the actual size of parameters)
  *
- * This function will return a unique ID the depends on the public key
+ * This function will return a unique ID that depends on the public key
  * parameters.  This ID can be used in checking whether a certificate
  * corresponds to the given private key.
  *
@@ -2519,7 +2544,7 @@ int ret;
       goto cleanup;
     }
   
-  algo = _gnutls_sign_get_hash_algorithm(ret);
+  algo = gnutls_sign_get_hash_algorithm(ret);
 
   ret = _gnutls_x509_get_signature (crq->crq, "signature", &signature);
   if (ret < 0)
