@@ -623,6 +623,17 @@ cleanup:
   return result;
 }
 
+static int is_printable(char p)
+{
+  if ((p >= 'a' && p <= 'z') || (p >= 'A' && p <= 'Z') ||
+      (p >= '0' && p <= '9') || p == ' ' || p == '(' || p == ')' ||
+      p == '+' || p == ',' || p == '-' || p == '.' || p == '/' || 
+      p == ':' || p == '=' || p == '?')
+    return 1;
+    
+  return 0;
+}
+
 /* This will encode and write the AttributeTypeAndValue field.
  * 'multi' must be (0) if writing an AttributeTypeAndValue, and 1 if Attribute.
  * In all cases only one value is written.
@@ -667,12 +678,12 @@ _gnutls_x509_encode_and_write_attribute (const char *given_oid,
 
       string_type = "printableString";
 
-      /* Check if the data is plain ascii, and use
+      /* Check if the data is ASN.1 printable, and use
        * the UTF8 string type if not.
        */
       for (i = 0; i < sizeof_data; i++)
         {
-          if (!isascii (data[i]))
+          if (!is_printable (data[i]))
             {
               string_type = "utf8String";
               break;
@@ -796,7 +807,7 @@ _gnutls_x509_write_attribute (const char *given_oid,
 /* Decodes an X.509 Attribute (if multi==1) or an AttributeTypeAndValue
  * otherwise.
  *
- * octet_string should be non (0) if we are to decode octet strings after
+ * octet_string should be non-zero if we are to decode octet strings after
  * decoding.
  *
  * The output is allocated and stored in value.
@@ -834,8 +845,10 @@ _gnutls_x509_decode_and_read_attribute (ASN1_TYPE asn1_struct,
   if (multi)
     _gnutls_str_cat (tmpbuffer, sizeof (tmpbuffer), "s.?1");    /* .values.?1 */
 
-  result =
-    _gnutls_x509_read_value (asn1_struct, tmpbuffer, value, octet_string);
+  if (octet_string)
+    result = _gnutls_x509_read_string (asn1_struct, tmpbuffer, value, RV_OCTET_STRING);
+  else
+    result = _gnutls_x509_read_value (asn1_struct, tmpbuffer, value);
   if (result < 0)
     {
       gnutls_assert ();
@@ -1073,7 +1086,7 @@ gnutls_x509_rdn_get (const gnutls_datum_t * idn,
  * @oid: an Object Identifier
  * @indx: In case multiple same OIDs exist in the RDN indicates which
  *   to send. Use 0 for the first one.
- * @raw_flag: If non (0) then the raw DER data are returned.
+ * @raw_flag: If non-zero then the raw DER data are returned.
  * @buf: a pointer to a structure to hold the peer's name
  * @sizeof_buf: holds the size of @buf
  *
