@@ -300,7 +300,7 @@ _gnutls_privkey_decode_ecc_key (const gnutls_datum_t * raw_key,
     }
 
   /* read the public key */
-  ret = _gnutls_x509_read_value(pkey_asn, "publicKey", &out, 2);
+  ret = _gnutls_x509_read_string(pkey_asn, "publicKey", &out, RV_BIT_STRING);
   if (ret < 0)
     {
       gnutls_assert();
@@ -561,7 +561,7 @@ failover:
 /**
  * gnutls_x509_privkey_import2:
  * @key: The structure to store the parsed key
- * @data: The DER or PEM encoded certificate.
+ * @data: The DER or PEM encoded key.
  * @format: One of DER or PEM
  * @password: A password (optional)
  * @flags: an ORed sequence of gnutls_pkcs_encrypt_flags_t
@@ -601,7 +601,7 @@ gnutls_x509_privkey_import2 (gnutls_x509_privkey_t key,
       ret = gnutls_x509_privkey_import_pkcs8(key, data, format, password, flags);
       if (ret < 0)
         {
-          if (format == GNUTLS_X509_FMT_PEM && password != NULL)
+          if (format == GNUTLS_X509_FMT_PEM)
             {
               int err;
               err = gnutls_x509_privkey_import_openssl(key, data, password);
@@ -1016,6 +1016,18 @@ int ret;
   return key->pk_algorithm;
 }
 
+static const char* set_msg(gnutls_x509_privkey_t key)
+{
+  if (key->pk_algorithm == GNUTLS_PK_RSA)
+    return PEM_KEY_RSA;
+  else if (key->pk_algorithm == GNUTLS_PK_DSA)
+    return PEM_KEY_DSA;
+  else if (key->pk_algorithm == GNUTLS_PK_EC)
+    return PEM_KEY_ECC;
+  else
+    return "UNKNOWN";
+}
+
 /**
  * gnutls_x509_privkey_export:
  * @key: Holds the key
@@ -1051,19 +1063,50 @@ gnutls_x509_privkey_export (gnutls_x509_privkey_t key,
       return GNUTLS_E_INVALID_REQUEST;
     }
 
-  if (key->pk_algorithm == GNUTLS_PK_RSA)
-    msg = PEM_KEY_RSA;
-  else if (key->pk_algorithm == GNUTLS_PK_DSA)
-    msg = PEM_KEY_DSA;
-  else if (key->pk_algorithm == GNUTLS_PK_EC)
-    msg = PEM_KEY_ECC;
-  else
-    msg = "UNKNOWN";
+  msg = set_msg(key);
 
   return _gnutls_x509_export_int (key->key, format, msg,
                                   output_data, output_data_size);
 }
 
+/**
+ * gnutls_x509_privkey_export2:
+ * @key: Holds the key
+ * @format: the format of output params. One of PEM or DER.
+ * @out: will contain a private key PEM or DER encoded
+ *
+ * This function will export the private key to a PKCS1 structure for
+ * RSA keys, or an integer sequence for DSA keys.  The DSA keys are in
+ * the same format with the parameters used by openssl.
+ *
+ * The output buffer is allocated using gnutls_malloc().
+ *
+ * If the structure is PEM encoded, it will have a header
+ * of "BEGIN RSA PRIVATE KEY".
+ *
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
+ *   negative error value.
+ *
+ * Since 3.1.3
+ **/
+int
+gnutls_x509_privkey_export2 (gnutls_x509_privkey_t key,
+                            gnutls_x509_crt_fmt_t format,
+                            gnutls_datum_t * out)
+{
+  const char *msg;
+
+  if (key == NULL)
+    {
+      gnutls_assert ();
+      return GNUTLS_E_INVALID_REQUEST;
+    }
+
+  msg = set_msg(key);
+
+  return _gnutls_x509_export_int2 (key->key, format, msg, out);
+}
+
 /**
  * gnutls_x509_privkey_sec_param:
  * @key: a key structure
@@ -1494,7 +1537,7 @@ gnutls_x509_privkey_verify_params (gnutls_x509_privkey_t key)
  * @output_data_size: holds the size of output_data (and will be
  *   replaced by the actual size of parameters)
  *
- * This function will return a unique ID the depends on the public key
+ * This function will return a unique ID that depends on the public key
  * parameters. This ID can be used in checking whether a certificate
  * corresponds to the given key.
  *
